interface AssetEventSchema {
    modelUuid: string;
    modelName: string;
    position: [number, number, number];
    rotation: [number, number, number];
    state: "idle" | "running" | "stopped" | "disabled" | "error";
}

interface TriggerSchema {
    triggerUuid: string;
    triggerName: string;
    triggerType: "onComplete" | "onStart" | "onStop" | "delay" | "onError";
    delay: number;
    triggeredAsset: {
        triggeredModel: { modelName: string, modelUuid: string };
        triggeredPoint: { pointName: string, pointUuid: string };
        triggeredAction: { actionName: string, actionUuid: string };
    } | null;
}

interface ConveyorPointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    action: {
        actionUuid: string;
        actionName: string;
        actionType: "default" | "spawn" | "swap" | "despawn";
        material: string;
        delay: number | "inherit";
        spawnInterval: number | "inherit";
        spawnCount: number | "inherit";
        triggers: TriggerSchema[];
    };
}

interface VehiclePointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    action: {
        actionUuid: string;
        actionName: string;
        actionType: "travel";
        material: string;
        unLoadDuration: number;
        loadCapacity: number;
        pickUpPoint: { x: number; y: number, z: number } | null;
        unLoadPoint: { x: number; y: number, z: number } | null;
        triggers: TriggerSchema[];
    };
}

interface RoboticArmPointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    actions: {
        actionUuid: string;
        actionName: string;
        actionType: "pickAndPlace";
        process: { startPoint: string; endPoint: string };
        triggers: TriggerSchema[];
    }[];
}

interface MachinePointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    action: {
        actionUuid: string;
        actionName: string;
        actionType: "process";
        processTime: number;
        swapMaterial: string;
        triggers: TriggerSchema[];
    };
}

interface StoragePointSchema {
    uuid: string;
    position: [number, number, number];
    rotation: [number, number, number];
    action: {
        actionUuid: string;
        actionName: string;
        actionType: "storage";
        materials: { materialName: string; materialId: string; }[];
        storageCapacity: number;
    };
}

interface ConveyorEventSchema extends AssetEventSchema {
    type: "transfer";
    speed: number;
    points: ConveyorPointSchema[];
}

interface VehicleSchemaEvent extends AssetEventSchema {
    type: "vehicle";
    speed: number;
    point: VehiclePointSchema;
}

interface RoboticArmSchemaEvent extends AssetEventSchema {
    type: "roboticArm";
    speed: number;
    point: RoboticArmPointSchema;
}

interface MachineSchemaEvent extends AssetEventSchema {
    type: "machine";
    point: MachinePointSchema;
}

interface StorageSchemaEvent extends AssetEventSchema {
    type: "storageUnit";
    point: StoragePointSchema;
}

type EventsSchema = ConveyorEventSchema | VehicleSchemaEvent | RoboticArmSchemaEvent | MachineSchemaEvent | StorageSchemaEvent | [];

type productsSchema = {
    productName: string;
    productId: string;
    eventsData: EventsSchema[];
}[]